using System;
using System.Collections.Generic;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;
using System.Windows.Threading;

namespace VLC.WPF
{
    public partial class VlcClient : System.Windows.Window
    {
        private static string BITMAP_BUFFER = "BitmapBuffer";
        private static string DIRECT_SHOW = "DirectShow";

        public VlcClient()
        {
            InitializeComponent();
            Video_Position.IsEnabled = true;
            Volume.Value = (double)VLC_Element.Volume;

            Source.Text = @"";

            VLC_Element.Watcher.WatcherUpdated += new EventHandler(Watcher_WatcherCallback);

            VideoOutputType_Selector.Items.Add(BITMAP_BUFFER);
            VideoOutputType_Selector.Items.Add(DIRECT_SHOW);
            VideoOutputType_Selector.SelectionChanged += new SelectionChangedEventHandler(VideoOutputType_Selector_SelectionChanged);
            VideoOutputType_Selector.SelectedIndex = 1;
            UpdateVideoOutputType();
        }

        private void UpdateVideoOutputType()
        {
            if (BITMAP_BUFFER.Equals(VideoOutputType_Selector.SelectedItem))
            {
                VLC_Element.VideoOutputType = VlcVideoOutputType.BitmapBuffer;
            }
            else if (DIRECT_SHOW.Equals(VideoOutputType_Selector.SelectedItem))
            {
                VLC_Element.VideoOutputType = VlcVideoOutputType.DirectShow;
            } 
        }

        private void VideoOutputType_Selector_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            UpdateVideoOutputType();            
        }

        private void VlcClient_Closing(object sender, System.ComponentModel.CancelEventArgs e)
        {
            VLC_Element.Dispose();
        }

        private void Play_Button_Click(object sender, RoutedEventArgs e)
        {
            VLC_Element.Source = Source.Text;
            System.Threading.Thread.Sleep(100);
            VLC_Element.Play2();
        }

        private void Pause_Button_Click(object sender, RoutedEventArgs e)
        {
            VLC_Element.Pause();
        }

        private void Stop_Button_Click(object sender, RoutedEventArgs e)
        {
            VLC_Element.Stop();
        }

        private void Volume_Changed(object sender, RoutedEventArgs e)
        {
            if (VLC_Element != null)
            {
                VLC_Element.Volume = (int)Volume.Value;
            }
        }

        private void Video_Position_Changing(object sender, RoutedEventArgs e)
        {
            if (VLC_Element != null)
            {
                VLC_Element.Watcher.HoldOn();
            }
        }

        private void Video_Position_Changed(object sender, RoutedEventArgs e)
        {
            if (VLC_Element != null)
            {
                VLC_Element.Position = (float)(Video_Position.Value / 100.0);
                VLC_Element.Watcher.Resume();
            }            
        }

        private void Window_Size_Changed(object sender, RoutedEventArgs e)
        {
            StretchSourceBlockToMaximum(); 
        }

        private void StretchSourceBlockToMaximum()
        {
            if (this.ActualWidth - 100 > 200)
            {
                Source.Width = this.ActualWidth - 100;
            }
        }

        private void Watcher_WatcherCallback(object sender, EventArgs e)
        {
            Video_Position.Value = Convert.ToInt32(VLC_Element.Watcher.VideoPosition * 100);
        }
    }
}