using System;
using System.Collections.Generic;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Windows.Input.StylusPlugIns;
using System.Windows.Threading;
using MediaBridge;

namespace VLC.WPF
{
    public partial class VlcElement : UserControl
    {
        private const string version = "0.0.2";

        private VlcVideoOutputType videoOutputType = VlcVideoOutputType.BitmapBuffer;
        private VlcWrapper vlc = null;
        private VlcWatcher watcher = null;
        private string source = null;

        private DispatcherTimer stopPlayTimer = new DispatcherTimer();
        private bool contentFinished = false;

        private Image videoImage = new Image();
        private VideoRendererElement videoRendererElement = new VideoRendererElement();

        public VlcElement()
        {
            InitializeComponent();
            VideoOutputController.VideoOutput = videoImage;
            VideoOutputController.VideoRenderer = videoRendererElement;
            VideoOutputController.VideoInitialized += new VideoOutputController.VideoEventHandler(VideoOutputController_VideoInitialized);
            VideoOutputType = VlcVideoOutputType.BitmapBuffer;
            vlc = new VlcWrapper();
            vlc.Initialize();

            watcher = new VlcWatcher(this);
            watcher.ContentFinished += new EventHandler(watcher_ContentFinished);

            stopPlayTimer.Interval = new TimeSpan(0, 0, 0, 0, 100);
            stopPlayTimer.Tick += new EventHandler(stopPlayTimer_Tick);
        }

        private void VideoOutputController_VideoInitialized()
        {
            watcher.OnContentStarted();
        }

        private void watcher_ContentFinished(object sender, EventArgs e)
        {
            System.Diagnostics.Debug.WriteLine("Content Finished!");
            contentFinished = true;
        }

        private void stopPlayTimer_Tick(object sender, EventArgs e)
        {
            stopPlayTimer.Stop();
            vlc.Stop();
            vlc.Play();
            watcher.Start();            
        }

        public VlcVideoOutputType VideoOutputType
        {
            get { return videoOutputType; }
            set 
            { 
                videoOutputType = value;
                VideoOutputController.VideoOutputType = (int)value;
                switch (videoOutputType)
                {
                    case VlcVideoOutputType.BitmapBuffer:
                        VideoBox.Child = videoImage;
                        break;
                    case VlcVideoOutputType.DirectShow:
                        VideoBox.Child = videoRendererElement;
                        break;
                }
            }
        }

        public static string Version
        {
            get { return version; }
        }

        public static string VlcVersion
        {
            get { return VlcWrapper.Version; }
        }

        public VlcWatcher Watcher
        {
            get { return watcher; } 
        }

        public string Source
        {
            get { return source; }
            set 
            {
                source = value;
                vlc.PlaylistClear();
                vlc.AddTarget(source);
            }
        }

        public int Volume
        {
            get { return vlc.VolumeGet; }
            set { vlc.VolumeSet(value); }
        }

        public float Position
        {
            get { return vlc.PositionGet; }
            set { vlc.PositionSet(value); }
        }

        public int Length
        {
            get { return vlc.LengthGet; }
        }

        public bool Fullscreen
        {
            get { return vlc.Fullscreen; }
            set { vlc.Fullscreen = value; }
        }

        public bool IsPlaying
        {
            get { return vlc.IsPlaying; }
        }

        public void AddTarget(string target)
        {
            vlc.AddTarget(target);
        }

        public void AddTarget(string target, string[] options)
        {
            vlc.AddTarget(target, options);
        }

        public void PlaylistClear()
        {
            vlc.PlaylistClear();
        }

        public void PlaylistNext()
        {
            vlc.PlaylistNext();
        }

        public void PlaylistPrevious()
        {
            vlc.PlaylistPrevious();
        }

        public void Play2()
        {
            if (contentFinished)
            {
                contentFinished = false;
                vlc.Play();
                stopPlayTimer.Start();
            }
            else
            {
                vlc.Play();
                watcher.Start();
            }
        }

        public void Play()
        {
            vlc.Play();
            watcher.Start();
        }

        public void Pause()
        {
            vlc.Pause();
            watcher.Pause();
        }

        public void Stop()
        {
            vlc.Stop();
            watcher.Stop();
        }

        public void SpeedFaster()
        {
            vlc.SpeedFaster();
        }

        public void SpeedSlower()
        {
            vlc.SpeedSlower();
        }

        public void VolumeMute()
        {
            vlc.VolumeMute();
        }

        public void Dispose()
        {
            watcher.Stop();
            vlc.Stop();
            vlc.Dispose();
            videoRendererElement.Dispose();
        }
    }
}